#! /usr/bin/python3.6

from pywintypes import com_error

from .exceptions import CATIAApplicationException

paper_sizes = [
    'catPaperLetter',  # [0]
    'catPaperLegal',  # [1]
    'catPaperA0',  # [2]
    'catPaperA1',  # [3]
    'catPaperA2',  # [4]
    'catPaperA3',  # [5]
    'catPaperA4',  # [6]
    'catPaperA',  # [7]
    'catPaperB',  # [8]
    'catPaperC',  # [9]
    'catPaperD',  # [10]
    'catPaperE',  # [11]
    'catPaperF',  # [12]
    'catPaperUser',  # [13]
]

drawing_orientation = [
    'catPaperPortrait',
    'catPaperLandscape',
    'catPaperBestFit',
]


class DrawingRoot:
    """

    """

    def __init__(self, catia):
        """

        :param catia: :class:`CATIAApplication()`
        """

        # the DrawingRoot COM object
        self.drawing_root = catia.catia.ActiveDocument.DrawingRoot

    @property
    def active_sheet(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property ActiveSheet( ) As DrawingSheet

            | Retrieves or sets the active sheet of the drawing.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example retrieves the active sheet in the drawing of the active document, supposed to be a drawing
            |   document.
            |       CATIA.ActiveDocument.DrawingRoot.ActiveSheet


        :return:
        """

        return self.drawing_root.ActiveSheet

    @property
    def parameters(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property Parameters( ) As Parameters (Read Only)

            | Returns the collection of parameters of the drawing document.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example retrieves in DrawingParameters the collection of parameters currently managed by the
            |   active document, supposed to be a drawing document.
            |       Dim DrawingParameters As Parameters
            |       Set DrawingParameters = CATIA.ActiveDocument.Parameters


        :return:
        """

        return self.drawing_root.Parameters

    @property
    def relations(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property Relations( ) As Relations (Read Only)

            | Returns the collection of relations of the drawing document.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example retrieves in DrawingRelations the collection of relations currently managed by the active
            |   document, supposed to be a drawing document.
            |       Dim DrawingRelations As Relations
            |       Set DrawingRelations = CATIA.ActiveDocument.Relations

        :return:
        """

        return self.drawing_root.Relations

    @property
    def sheets(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property Sheets( ) As DrawingSheets (Read Only)

            |Returns the collection of drawing sheets of the drawing document.
            |Warning: This method is not available with 2D Layout for 3D Design.
            |Example:
            |   This example retrieves in SheetCollection the collection of sheets currently managed by the active
            |   document, supposed to be a drawing document.
            |       Dim SheetCollection As DrawingSheets
            |       Set SheetCollection = CATIA.ActiveDocument.Sheets


        :return:
        """

        _sheets = list()
        com_sheets = self.drawing_root.Sheets

        for item in range(com_sheets.Count):
            _sheets.append(DrawingSheet(com_sheets.Item(item + 1)))

        return _sheets

    @property
    def standard(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property Standard( ) As CatDrawingStandard

            | Returns or sets the drawing standard of the drawing document.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example sets the drawing standard of the active document, supposed to be a drawing document, to
            |   ISO.
            |       CATIA.ActiveDocument.Standard = catISO


            .. note::
            CAA V5 Visual Basic help

            | CatDrawingStandard (Enumeration)
            | enum CatDrawingStandard {
            |   catANSI,
            |   catISO,
            |   catJIS
            | }
            |
            | The standard for the drawing document.
            |
            | Values:
            |     catANSI
            | The standard is ANSI
            |     catISO
            | The standard is ISO
            |     catJIS
            | The standard is JIS


        :return:
        """

        standards = ['ANSI', 'ISO', 'JIS']

        return standards[self.drawing_root.Standard]

    def isolate(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Sub Isolate( )

            | Isolates all the drawing views of all the drawing sheets of the drawing document.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example isolates all the drawing views of all the drawing sheets of the active document, supposed
            |   to be a drawing document.
            |       CATIA.ActiveDocument.Isolate

        :return:
        """

        self.drawing_root.Isolate()

    def update(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Sub Update( )

            | Updates all the drawing sheets of the drawing document.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example updates the active document, supposed to be a drawing document.
            |       CATIA.ActiveDocument.Update

        :return:
        """

        self.drawing_root.Update()

    def reorder_sheets(self, new_sheet_order):
        """
        .. note::
            CAA V5 Visual Basic help

            Sub reorder_Sheets( CATSafeArrayVariant  iOrderedSheets)

            | Changes the positions of the sheets in this drawing according to the given ordered list. iOrderedSheets
            | is the result of a permutation applied to the list of all the sheets of this drawing, with the following
            | constraint: For every non-detail sheet, there is not any detail sheet appearing before in iOrderedSheets.
            | Example:
            |   This example inverts the sheet order of a drawing made of exactly two regular sheets.
            |       Set drwsheets = CATIA.ActiveDocument.Sheets
            |       Set drwsheetsorder =  CATIA.ActiveDocument.DrawingRoot
            |       Set sheet1 = drwsheets.item(1)
            |       Set sheet2 = drwsheets.item(2)
            |       newsheetorder = Array(sheet2, sheet1)
            |       drwsheetsorder.reorder_Sheets(newsheetorder)


        :param new_sheet_order:
        :return:
        """

        self.drawing_root.reorder_Sheets(new_sheet_order)


class DrawingSheet:
    """

    """

    def __init__(self, drawing_sheet):
        """

        :param drawing_sheet: the DrawingSheet COM object.
        """

        self.sheet = drawing_sheet

    @property
    def name(self):
        return self.sheet.Name

    @name.setter
    def name(self, name):
        self.sheet.Name = name

    @property
    def orientation(self):
        """
        CAA V5 Visual Basic help

        Property Orientation( ) As CatPaperOrientation

        | Returns or sets the paper orientation.
        | Example:
        |   This example sets the paper orientation for the MySheet drawing sheet to catPaperLandscape.
        |       MySheet.Orientation = catPaperLandscape

        :return:
        """

        return drawing_orientation[self.sheet.Orientation]

    @orientation.setter
    def orientation(self, drawing_orientation):
        """

        :param int drawing_orientation: ['catPaperPortrait', 'catPaperLandscape', 'catPaperBestFit']
        :return:
        """
        self.sheet.Orientation = drawing_orientation

    @property
    def scale(self):
        return self.sheet.Scale

    @scale.setter
    def scale(self, scale):
        self.sheet.Scale = scale

    @property
    def paper_size(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property PaperSize( ) As CatPaperSize

            | Returns or sets the paper size.
            | Example:
            |   This example sets the page size for the MySheet drawing sheet to catPaperA4.
            |       MySheet.PaperSize = catPaperA4


        .. note::
            CAA V5 Visual Basic help

            | CatPaperSize (Enumeration)
            | enum CatPaperSize {
            |   catPaperLetter,
            |   catPaperLegal,
            |   catPaperA0,
            |   catPaperA1,
            |   catPaperA2,
            |   catPaperA3,
            |   catPaperA4,
            |   catPaperA,
            |   catPaperB,
            |   catPaperC,
            |   catPaperD,
            |   catPaperE,
            |   catPaperF,
            |   catPaperUser
            | }
            |
            | Paper size.
            | It used by the PageSetup object for document printing.
            | Values:
            |   catPaperLetter
            |       The US Letter format (8.5 x 11")
            |   catPaperLegal
            |       The Legal format (8.5 x 14")
            |   catPaperA0
            |       The A0 ISO format (841 x 1189 mm)
            |   catPaperA1
            |       The A1 ISO format (594 x 841 mm)
            |   catPaperA2
            |       The A2 ISO format (420 x 594 mm)
            |   catPaperA3
            |       The A3 ISO format (297 x 420 mm)
            |   catPaperA4
            |       The A4 ISO format (210 x 297 mm)
            |   catPaperA
            |       The A ANSI format (8.5 x 11")
            |   catPaperB
            |       The B ANSI format (11 x 17")
            |   catPaperC
            |       The C ANSI format (17 x 22")
            |   catPaperD
            |       The D ANSI format (22 x 34")
            |   catPaperE
            |       The E ANSI format (34 x 44")
            |   catPaperF
            |       The F ANSI format (28 x 40")
            |   catPaperUser
            |       The customized format


        :return:
        """

        return self.sheet.PaperSize

    @paper_size.setter
    def paper_size(self, paper_size):

        try:
            self.sheet.PaperSize = paper_size
        except com_error:
            raise CATIAApplicationException('Ensure paper_size is available for the drawing standard.')

    @property
    def project_method(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property ProjectionMethod( ) As CatSheetProjectionMethod

            | Returns or sets the sheet projection mode .
            | Example:
            |   This example sets the projection mode of the MySheet drawing sheet to catFirstAngle.
            |       MySheet.ProjectionMethod = catFirstAngle


        .. note::
            CAA V5 Visual Basic help

            | CatSheetProjectionMethod (Enumeration)
            | enum CatSheetProjectionMethod {
            |   catFirstAngle,
            |   catThirdAngle
            | }
            |
            | Sheet projection Method.
            |
            | Values:
            |   catFirstAngle
            |       The views are created using first angle standard
            |   catThirdAngle
            |       The views are created using third angle standard

        :return:
        """

        values = ['FirstAngle', 'ThirdAngle']

        return values[self.sheet.ProjectionMethod]

    @property
    def views(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property Views( ) As DrawingViews (Read Only)

            | Returns the drawing view collection of the drawing sheet.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example retrieves in ViewCollection the collection of views of the MySheet drawing sheet.
            |       Dim ViewCollection As DrawingViews
            |       Set ViewCollection = MySheet.Views


        :return:
        """

        return Views(self.sheet.Views)


class Views:

    def __init__(self, views):
        """

        .. note::
            CAA V5 Visual Basic help

            A collection of all the drawing views currently managed by a drawing sheet in a drawing document.


        :param views: the DrawingViews COM object
        """

        self.views = views

    @property
    def active_view(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property ActiveView( ) As DrawingView (Read Only)

            | Returns the active drawing view of the active drawing sheet.
            | Warning: This method is not available with 2D Layout for 3D Design.
            |   Example:
            |   The following example retrieves in ViewToWorkIn the active drawing view in the DrawingSheets collection
            |   of the document named CATDrawing1
            |       Dim MyDrawingDoc As Document
            |       Set MyDrawingDoc = CATIA.Documents.Item("CATDrawing1")
            |       Dim ViewToWorkIn As DrawingView
            |       Set ViewToWorkIn = MyDrawingDoc.Sheets.ActiveSheet.DrawingViews.ActiveView


        :return:
        """

        return DrawingView(self.views.ActiveView)

    def view_list(self):
        """

        :return:
        """

        # todo: return a list of views.

    def get_view_by_name(self, view_name):
        """

        :param str view_name:
        :return:
        """

        return DrawingView(self.views.Item(view_name))


class DrawingView:
    """

    """

    def __init__(self, drawing_view):
        """
        .. note::
            CAA V5 Visual Basic help

            Represents a drawing view in a drawing sheet.

            | The drawing view is included in a drawing sheet and contains texts,leaders, dimensions, arrows, pictures,
            | tables, 2D Geometry and 2D component.
            | Warning: This interface is not available with 2D Layout for 3D Design.


        :param drawing_view: the DrawingView COM object
        :return:
        """

        self.drawing_view = drawing_view

    @property
    def name(self):
        return self.drawing_view.Name

    @name.setter
    def name(self, name):
        self.drawing_view.Name = name

    @property
    def scale(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property Scale( ) As double

            | Returns or sets the scale of the drawing view.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example sets the scale of the MyView drawing view to 0.5.
            |       MyView.Scale = 0.5


        :return: float
        """
        return self.drawing_view.Scale

    @scale.setter
    def scale(self, scale):
        self.drawing_view.Scale = scale

    def activate(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Sub Activate( )

            | Activates the drawing view. Activating a drawing view means that this drawing view is the one on which
            | the end-user is now working.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example activates the ViewToWorkOn drawing view.
            |       ViewToWorkOn.Activate()



        :return:
        """

        self.drawing_view.Activate()

    @property
    def generative_view_behaviour(self):
        """
        .. note::
            CAA V5 Visual Basic help

            Property GenerativeBehavior( ) As DrawingViewGenerativeBehavior (Read Only)

            | Returns the generative behavior of the drawing view.
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example retrieves in MyViewGenBehavior the generative behavior of the MyView drawing view.
            |       Dim MyViewGenBehavior As DrawingViewGenerativeBehavior
            |       Set MyViewGenBehavior = MyView.GenerativeBehavior


        :return:
        """

        return self.drawing_view.GenerativeBehavior

    def get_view_name(self):
        """
        .. note::
            CAA V5 Visual Basic help

            | Sub GetViewName( CATBSTR  iViewNamePrefix,
            |                  CATBSTR  iViewNameIdent,
            |                  CATBSTR  iViewNameSuffix)
            |
            | Returns the prefix, the ident and the suffix of the name of the drawing view. The method returns an error
            | in case of 2D component reference.
            | Note: Prefix of drawing view can be also retrieved across name property defined in CATIABase
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example gets the prefix, the ident, and the suffix of the name of the MyView drawing view
            |       Dim MyPrefix, MyIdent, MySuffix As CATBSTR
            |       MyView.GetViewName (MyPrefix, MyIdent, MySuffix)


        :return: tuple(prefix, ident, suffix)
        """
        return self.drawing_view.GetViewName()

    def set_view_name(self, prefix, ident, suffix):
        """
        .. note::
            CAA V5 Visual Basic help

            | Sub SetViewName( CATBSTR  iViewNamePrefix,
            |                  CATBSTR  iViewNameIdent,
            |                  CATBSTR  iViewNameSuffix)
            |
            | Sets the prefix, the ident and the suffix of the name of the drawing view. The method returns an error in
            | case of 2D component reference.
            | Note: Prefix of drawing view can be also modified across name property defined in CATIABase
            | Warning: This method is not available with 2D Layout for 3D Design.
            | Example:
            |   This example sets the prefix, the ident, and the suffix of the name of the MyView drawing view
            |   respectively to "MyPrefix", "MyIdent", and "MySuffix".
            |       MyView.SetViewName ("MyPrefix", "MyIdent", "MySuffix")


        .. warning::
            The view name will not update in the view tree. But if part is saved and reopened the view name is updated.


        :param str prefix:
        :param str ident:
        :param str suffix:
        :return:
        """

        self.drawing_view.SetViewName(prefix, ident, suffix)



#! /usr/bin/python3.6

import os
from pywintypes import com_error
import warnings

from .exceptions import CATIAApplicationException
from .part import Part
from .product import Product


class Documents:
    """
    The Documents object is used to access multiple open documents in the catia session.

    Usage::

        from pycatia import CATIAApplication
        catia = CATIAApplication()
        documents = Document(catia.catia)

    .. note::
        CAA V5 Visual Basic help

        A collection of all the Document objects currently managed by the application.
        These documents belong to one of the following types:

            PartDocument,
            ProductDocument,
            Drawing.


    :param catia: CATIA COM object

    """

    def __init__(self, catia):

        self.catia = catia
        self.documents = catia.Documents

    def add(self, document_type):
        """
        .. note::
            CATIA V5 Visual Basic Help

            Func Add( CATBSTR  docType) As Document

            Creates a Document object and adds it to the documents collection. This document becomes the active one,
            and a window is created to accomodate it which becomes the active window.
            | Parameters:
            |   docType
            | The type of the document to create, chosen among:
            |   Part
            |       For PartDocument
            |   Product
            |       For ProductDocument
            |   Drawing
            |       For Drawing
            |   Returns:
            |       The created document
            | Example:
            | The following example creates a PartDocument document in the collection retrieved in PartDoc.
            |   Dim PartDoc As Document
            |   Set PartDoc = Documents.Add("Part")

        :param document_type:
        :return:
        """

        document_types = ['Part', 'Product', 'Drawing']
        if document_type not in document_types:
            raise ValueError(f'Document type must be in [{document_types}]')

        self.documents.Add(document_type)

    def new_from(self, file_name):
        """

        .. note::
            CAA V5 Visual Basic help

            Creates a new document from a document stored in a file. Role: Reads a document stored in a file and
            creates a new document containing the resulting data, adds the new document to the document collection,
            displays it in a new window, adds the window to the window collection and activates both the document and
            the window.

            | Parameters:
            |   The name of the file containing the document.
            | Returns:
            |   The created document.
            | Example:
            | The following example creates a new Doc document from the contents of the FileToRead file.
            |   FileToRead = "e:\\users\\psr\\Parts\\ThisIsANicePart.CATPart"
            |   Dim Doc As Document
            |   Set Doc = Documents.NewFrom(FileToRead)


        :param str file_name:
        :return:

        """

        if not os.path.isfile(file_name):
            raise FileNotFoundError(f'Could not find file {file_name}.')

        # get the full path to the file
        file_name = os.path.abspath(file_name)

        return self.documents.NewFrom(file_name)

    def open(self, file_name):
        """
        Open CATIA document `file_name` in current CATIA session.

        .. note::
            CAA V5 Visual Basic help

            Func Open( CATBSTR  iFileName) As Document

            Opens a document stored in a file. Reads a document stored in a file, displays it in a new window, adds the
            document to the documents collection and the window to the windows collection, and makes both the document
            and the window the active ones.

            | Parameters:
            |   iFileName:
            |       The name of the file containing the document
            |   Returns:
            |       The retrieved document

            | Example:
            | The following example opens the Doc document contained in the FileToOpen file.
            | FileToOpen = "e:/users/psr/Parts/ThisIsANicePart.CATPart"
            | Dim Doc As Document
            | Set Doc = Documents.Open(FileToOpen)

        :param str file_name: full path to catia file.
        """

        if not os.path.isfile(file_name):
            raise FileNotFoundError(f'Could not find file {file_name}.')

        # get the full path to the file
        file_name = os.path.abspath(file_name)
        self.documents.Open(file_name)

    def num_open(self):
        """
        Returns the number of open documents.

        .. warning::

            The COM object can return the incorrect number of documents open. After a document is closed CATIA can keep
            the linked document `ABQMaterialPropertiesCatalog.CATfct` open.

        :return: int()
        """

        # for i in range(0, self.documents.Count):
        #     print(self.documents.Item(i + 1).Name)

        warning_string = (
            'The COM object can return the incorrect number of documents open. \n'
            'For example, after a CATPart document is closed CATIA can keep'
            'the linked document `ABQMaterialPropertiesCatalog.CATfct` loaded in the session.'
        )

        warnings.warn(warning_string)
        return self.documents.Count


class Document:
    """
    The Document object is used to access the currently active document in the catia session.

    If the document fails to activate try closing CATIA and killing any existing COM object
    processes in task manager.

    .. note::

        CAA V5 Visual Basic help

        The Document is the object which stores your pycatia data on disk. A document is either created
        empty using the File->New command or using the Add method of the Documents collection, or opened
        from a file using the File->Open command or using the Open method of the Documents collection.
        CATIA provides access to four document types: the PartDocument, the ProductDocument, the
        DrawingDocument and the AnalysisDocument. The Document abstract object gathers the properties and
        methods common to all actual document types. When a document is created or opened, it is
        automatically set as the active document and displayed in a window which automatically becomes
        the active window. A document aggregates the current object or set of objects in the Selection
        object, and Cameras, a camera collection.


    :param catia: CATIA COM object.
    """

    def __init__(self, catia):

        try:
            self.document = catia.ActiveDocument
        except com_error:
            message = "Could not activate document. Is a document open?"
            raise CATIAApplicationException(message)

    @property
    def is_part(self):
        """
        Determine whether the active document is a CATPart.

        :return: bool
        """
        try:
            if self.part().part:
                return True
        except AttributeError:
            return False

    @property
    def is_product(self):
        """
        Determine whether the active document is a CATProduct.

        :return: bool
        """

        if self.product().is_catproduct():
            return True
        return False

    @property
    def is_saved(self):
        """
        Returns true if document is saved.

        .. note::
            CAA V5 Visual Basic help

            Property Saved( ) As boolean (Read Only)

            Returns whether the document has been modified, and thus needs to be saved.
            This happens when the document has changed since either its creation or its last save.
            True if the document has not been changed: the document doesn't need to be saved.
            False if the document has been changed: the document needs to be saved.

            | Example:
            | This example retrieves in HasChanged whether the Doc document needs to be saved.
            | HasChanged = NOT Doc.Saved

        :return: bool
        """

        return self.document.Saved

    @property
    def name(self):
        """

        :return: str - document name.
        """

        return self.document.Name

    @property
    def full_name(self):
        """

        .. note::
            CAA V5 Visual Basic help

            Property FullName( ) As CATBSTR (Read Only)

            | Returns the document's full file name, including its path.
            | Example:
            | This example retrieves in DocFullName the Doc document's full file name.
            |     DocFullName = Doc.FullName
            |
            | The returned value is like this:
            |     e:\\users\\psr\\Parts\\MyNicePart.CATPart


        :return: str - full path document name
        """

        return self.document.FullName

    @property
    def path(self):
        """

        .. note::
            CAA V5 Visual Basic help

            Property Path( ) As CATBSTR (Read Only)

            | Returns the document's file path.
            | Example:
            | This example retrieves in DocPath the path where the Doc document is stored.
            |     DocPath = Doc.Path
            |
            | The returned value is like this:
            |     e:\\users\\psr\\Parts


        :return: str - path to document
        """

        return self.document.Path

    def export_data(self, filename, filetype):

        """
        .. note::
            CAA V5 Visual Basic help

            Sub ExportData( CATBSTR  fileName, CATBSTR  format)

            | Exports the data contained in the document to another format.
            | Parameters:
            |   fileName
            |       The name of the exported file
            |   format
            |       The name of the format
            | Example:
            |   This example writes the Doc document in the IGES format under the IGESDoc name.
            |       Doc.ExportData("IGESDoc", "igs")


        :param str filename: filename including full path.
        :param str filetype: filetype is the extension of required filetype. The filetype must be supported by CATIA.
        :return:
        """

        self.document.ExportData(filename, filetype)

    def product(self):
        """
        :return: :class:`Product()`
        """

        return Product(self.document.Product)

    def part(self):
        """
        :return: :class:`Part()`
        """

        return Part(self.document.Part)

    def activate(self):
        """
        Activates the document

        .. note::
            CAA V5 Visual Basic help

            Sub Activate( )

            Activates the document. Activating a document means that this document is the one on which the end user is
            now working on. This document possibly reconfigures the menu bar and toolbars with its own commands if its
            type is different from the type of the previous active document. The first window in the window collection
            which contains this document becomes the active one.

            Example:
            | This example activates the Doc document.
            | Doc.Activate()

        """

        self.document.Activate()

    def close(self):
        """
        Closes the current document.

        .. note::
            CAA V5 Visual Basic help

            Sub Close( )

            Closes the document. This closes all the windows displaying the document. If the document needs to be saved,
            the end user is prompted whether to save the document, or to close it anyway.

            | Example:
            | This example closes the Doc document
            |  Doc.Close()

        """

        self.document.Close()

    def save(self):
        """
        Save the current document.

        .. note::
            CAA V5 Visual Basic help

            Sub Save( )

            Saves the document.
            | Example:
            | This example saves the Doc document.
            | Doc.Save()

        """

        self.document.Save()

    def save_as(self, file_name):
        """
        Save the document to a new name.

        .. note::
            CAA V5 Visual Basic help

            Sub SaveAs( CATBSTR  fileName)

            Saves the document with another name.
            |  Parameters:
            |     fileName
            |         The name to assign to the document
            | Example:
            | This example saves the Doc document with the NewName name.
            | Doc.SaveAs("NewName")

        :param str file_name: full pathname to new file_name
        """

        file_name = os.path.abspath(file_name)
        if os.path.isfile(file_name):
            raise FileExistsError(f'File: {file_name} already exists.')
        self.document.SaveAs(file_name)

    @staticmethod
    def search_for_items(document, selection_objects):
        """

        # todo: This search is currently restricted to GSD objects only.

        Selection objects is a list of items to search for.
        Example: selection_objects = ['Point', 'Line']

        Example query string to search for all lines and points
        "('Generative Shape Design'.Point + 'Generative Shape Design'.Line),in"

        :param document:
        :param list selection_objects:
        :return Selected Automation Object:
        """

        gsd_items = [
            'Point',
            'Line'
        ]

        query_string = str()
        # build query string

        for counter, item in enumerate(selection_objects):
            boolean = str()
            if counter > 0 and not counter == len(selection_objects):
                boolean = ' + '
            if item in gsd_items:
                query_string = f"{query_string}{boolean}'Generative Shape Design'.{item}"

        query_string = f"({query_string}),in"

        selection = document.document.Selection
        selection.Search(query_string)

        selected = list()
        for i in range(0, selection.Count):
            selected.append(selection.Item(i + 1).Value)

        return selected

    def __repr__(self):
        return f'Document() name: {self.name}'



#! /usr/bin/python3.6

import os

from .catia_application import CATIAApplication
from .csv_tools import create_points
from .csv_tools import csv_reader
from .context import CATIADocHandler
from .document import Document
from .document import Documents
from .drawing import DrawingRoot
from .drawing import DrawingSheet
from .drawing import Views
from .drawing import DrawingView
from .drawing import paper_sizes
from .drawing import drawing_orientation
from .exceptions import CATIAApplicationException
from .general_functions import create_measurable
from .hybridshapefactory import HybridShapeFactory
from .measurable import CATIAMeasurable
from .part import Part
from .product import Product
from .workbenches import create_spa_workbench


__author__ = 'Paul Bourne'
__author_email = 'evereux@gmail.com'
__description__ = 'A python module to access the CATIA Measurable object.'
__name__ = "pycatia"
__version__ = "0.1.5.1"
__url__ = "https://github.com/evereux/pycatia"

name = __name__

macro_path = os.path.join(os.path.realpath(__file__), r'macros')
